; Remove lower border in 60 Hz
; For "TheNameOfTheGame" on Atari-Forum.com
; v1
;
; Builds with Devpac 3 (gen.ttp) and likely vasm as well
;
; 2022-09-02
; ae@dhs.nu

		output	.tos

		section	text

start:		pea	demo				;Run demo in supervisor
		move.w	#$26,-(sp)			;supexec()
		trap	#14				;
		addq.l	#6,sp				;

		clr.w	-(sp)				;pterm()
		trap	#1				;



demo:		bsr	save_and_setup_hardware
		bsr	copy_picture_to_screen
		bsr	save_mfp
		bsr	setup_mfp


;--------------- MAINLOOP ------------------------------
.mainloop:	tst.w	vblcount			;Wait VBL
		beq.s	.mainloop			;
		clr.w	vblcount			;

		cmp.b	#$39,$fffffc02.w		;Space?
		bne	.mainloop			;

		bsr	restore
		rts


;-------------- VBL -------------------------------------
vbl:		movem.l	d0-a6,-(sp)

		addq.w	#1,vblcount

		move.l	scraddr,d0
		lsr.w	#8,d0
		move.l	d0,$ffff8200.w

		movem.l	pi1+2,d0-d7
		movem.l	d0-d7,$ffff8240.w

		move.w	sr,d0
		move.w	#$2700,sr
		move.l	#lowborder_tb,$120.w
		clr.b	$fffffa1b.w			;Timer B control (stop)
		bset	#0,$fffffa07.w			;Interrupt enable A (Timer B)
		bset	#0,$fffffa13.w			;Interrupt mask A (Timer B)
		move.b	#200,$fffffa21.w		;Timer B data (number of scanlines to next interrupt)
		move.b	#8,$fffffa1b.w			;Timer B control (event mode (HBL))
		bclr	#3,$fffffa17.w			;Automatic end of interrupt
		move.w	d0,sr

		movem.l	(sp)+,d0-a6
dummy:		rte



;-------------- TIMER B LOWER BORDER -------------------
lowborder_tb:
		move.b	#2,$ffff820a.w			;50 Hz
		dcb.w	2,$8080
		clr.b	$ffff820a.w			;60 Hz
		rte


;--------------- SERVICE ROUTINES ----------------------
save_and_setup_hardware:
		lea	save_screenadr,a0		;Save old screen address
		move.b	$ffff8201.w,(a0)+		;
		move.b	$ffff8203.w,(a0)+		;
		move.b	$ffff820d.w,(a0)+		;

		movem.l	$ffff8240.w,d0-d7		;Save old palette
		movem.l	d0-d7,save_pal			;

		move.l	#screen+256,d0			;Align new screen address to 64k
		clr.b	d0				;
		move.l	d0,scraddr			;

		lsr.w	#8,d0				;Set new screen address
		move.l	d0,$ffff8200.w			;

		move.b	$ffff8260.w,save_res		;Save old resolution
		clr.b	$ffff8260.w			;Set low resolution

		move.b	$ffff820a.w,save_freq		;Save old refresh
		clr.b	$ffff820a.w			;Set 60 Hz

		move.b	$ffff820f.w,save_lw		;Save old linewidth

		move.b	#$12,$fffffc02.w		;Kill mouse
		rts

save_mfp:	move.w	sr,d0
		move.w	#$2700,sr			;Stop all interrupts
		lea	save_vect_mfp,a0
		move.l	$70.w,(a0)+			;Save old VBL
		move.l	$68.w,(a0)+			;Save old HBL
		move.l	$134.w,(a0)+			;Save old Timer A
		move.l	$120.w,(a0)+			;Save old Timer B
		move.l	$114.w,(a0)+			;Save old Timer C
		move.l	$110.w,(a0)+			;Save old Timer D
		move.l	$118.w,(a0)+			;Save old ACIA
		move.b	$fffffa07.w,(a0)+
		move.b	$fffffa09.w,(a0)+
		move.b	$fffffa13.w,(a0)+
		move.b	$fffffa15.w,(a0)+
		move.b	$fffffa19.w,(a0)+
		move.b	$fffffa1b.w,(a0)+
		move.b	$fffffa1f.w,(a0)+
		move.b	$fffffa21.w,(a0)+
		clr.b	$fffffa07.w			;Interrupt enable A (Timer-A & B)
		clr.b	$fffffa13.w			;Interrupt mask A (Timer-A & B)
		clr.b	$fffffa09.w			;Interrupt enable B (Timer-C & D)
		clr.b	$fffffa15.w			;Interrupt mask B (Timer-C & D)
		move.w	d0,sr				;Interrupts back
		rts

setup_mfp:	move.w	sr,d0
		move.w	#$2700,sr			;Stop all interrupts
		lea	dummy(pc),a0
		move.l	#vbl,$70.w			;Install our own VBL
		move.l	a0,$68.w			;Install our own HBL
		move.l	a0,$134.w			;Install our own Timer A
		move.l	a0,$120.w			;Install our own Timer B
		move.l	a0,$114.w			;Install our own Timer C
		move.l	a0,$110.w			;Install our own Timer D
		move.l	a0,$118.w			;Install our own ACIA
		move.w	d0,sr				;Interrupts back
		rts

restore:	move.w	sr,d0
		move.w	#$2700,sr			;Stop all interrupts
		lea	save_vect_mfp,a0
		move.l	(a0)+,$70.w			;Restore old VBL
		move.l	(a0)+,$68.w			;Restore old HBL
		move.l	(a0)+,$134.w			;Restore old Timer A
		move.l	(a0)+,$120.w			;Restore old Timer B
		move.l	(a0)+,$114.w			;Restore old Timer C
		move.l	(a0)+,$110.w			;Restore old Timer D
		move.l	(a0)+,$118.w			;Restore old ACIA
		move.b	(a0)+,$fffffa07.w
		move.b	(a0)+,$fffffa09.w
		move.b	(a0)+,$fffffa13.w
		move.b	(a0)+,$fffffa15.w
		move.b	(a0)+,$fffffa19.w
		move.b	(a0)+,$fffffa1b.w
		move.b	(a0)+,$fffffa1f.w
		move.b	(a0)+,$fffffa21.w
		move.w	d0,sr				;Interrupts back

		move.w	#37,-(sp)			;vsync()
		trap	#14				;
		addq.l	#2,sp				;

		clr.b	$ffff8265.w			;Reset hscroll
		clr.b	$ffff820f.w			;Reset modulo
		move.b	save_res,$ffff8260.w		;Restore old resolution
		move.b	save_freq,$ffff820a.w		;Restore old refresh

		lea	save_screenadr,a0		;Restore old screen address
		move.b	(a0)+,$ffff8201.w		;
		move.b	(a0)+,$ffff8203.w		;
		move.b	(a0)+,$ffff820d.w		;

		move.b	save_lw,$ffff820f.w		;Restore old linewidth

		movem.l	save_pal,d0-d7			;Restore old palette
		movem.l	d0-d7,$ffff8240.w		;

		move.b	#$8,$fffffc02.w			;Enable mouse
		rts

copy_picture_to_screen:
		;first 200 lines
		move.l	scraddr,a0
		lea	pi1+34,a1
		move.w	#160*200/4-1,d7
.loop:		move.l	(a1)+,(a0)+
		dbra	d7,.loop

		;some extra lines in the low border
		;(Hatari 2.4.1 shows 23 extra lines)
		lea	pi1+34,a1
		move.w	#160*50/4-1,d7
.loop2:		move.l	(a1)+,(a0)+
		dbra	d7,.loop2

		rts


		section	data

pi1:		incbin	'pic.pi1'
		even

		section	bss

scraddr:	ds.l	1
vblcount:	ds.w	1
screen:		ds.b	160*250+256
save_pal:	ds.w	16
save_screenadr:	ds.l	1
save_vbl:	ds.l	1
save_hbl:	ds.l	1
save_ta:	ds.l	1
save_tb:	ds.l	1
save_tc:	ds.l	1
save_td:	ds.l	1
save_acia:	ds.l	1
save_vect_mfp:	ds.b	28+8
save_lw:	ds.b	1
save_res:	ds.b	1
save_freq:	ds.b	1
		even

		end
